Package: voice
Type: Package
Title: Speaker Recognition, Voice Analysis and Mood Inference via Music
        Theory
Version: 0.5.6
Date: 2026-05-13
Authors@R: person("Zabala", "Filipe J.", email = "filipezabala@gmail.com", role = c("cre","aut"))
Maintainer: Zabala Filipe J. <filipezabala@gmail.com>
Description: Provides tools for audio data analysis, including feature extraction, pitch detection, and speaker identification. Designed for voice research and signal processing applications.
License: GPL-3
Depends: R (>= 4.1.0)
Imports: arrangements, dplyr, ggplot2, htmltools, httr, httr2, R.utils,
        reticulate, seewave, tabr, tibble, tidyselect, tuneR, wrassp,
        zoo
Suggests: gm, knitr, tidyverse
URL: https://github.com/filipezabala/voice
BugReports: https://github.com/filipezabala/voice/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-13 19:42:06 UTC; filipezabala
Author: Zabala Filipe J. [cre, aut]
Repository: CRAN
Date/Publication: 2026-05-13 20:00:02 UTC
Built: R 4.6.0; ; 2026-05-13 20:25:03 UTC; unix
