# Generated by roxygen2: do not edit by hand

export(assign_notes)
export(audio_time)
export(check_chords)
export(cut_audio)
export(diarize)
export(duration)
export(embed_audio)
export(embed_video)
export(enrich_rttm)
export(expand_model)
export(extract_features)
export(feat_summary)
export(get_bit)
export(get_dur)
export(get_left)
export(get_right)
export(get_samp.rate)
export(get_tbeg)
export(get_tdur)
export(interp)
export(interp_df)
export(interp_mc)
export(is_mono)
export(mp3)
export(mp4)
export(notes)
export(notes_freq)
export(piano_plot)
export(png)
export(read_rttm)
export(rm0)
export(smooth_df)
export(splitw)
export(spn2abc)
export(tag)
export(transcribe)
export(write_list)
import(reticulate)
importFrom(dplyr,"%>%")
importFrom(dplyr,mutate_each)
importFrom(dplyr,select)
importFrom(stats,median)
importFrom(stats,quantile)
