# `voice`

General tools for voice analysis. 

The `voice` package is being developed to be an easy-to-use set of tools to deal with audio analysis in R. It provides a free and user-friendly toolkit for audio analysis, enabling researchers to extract, tag, and analyze voice data efficiently. It supports the extraction of audio features, enrichment of structured datasets with audio summaries, and automatic identification of spoken segments—while introducing novel features. It also allows audio analysis based on musical theory, associating frequencies with musical notes arranged in a score via [`gm`](https://github.com/flujoo/gm) package. 

## Basic installation
`devtools::install_github('filipezabala/voice')`

More details may be found at 
- https://filipezabala.com/voicegnette/
- https://cran.r-project.org/package=voice 

## Issues
If you want to contribute, report bugs or request new features, use the 'Issues' tab on Github.
